﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using System.Threading;
using System.Threading.Tasks;
using Tuitio.Application.Services;
using Tuitio.PublishedLanguage.Dto;

namespace Tuitio.Application.CommandHandlers
{
    public class AccountLogoutHandler
    {
        public record Command(string Token) : IRequest<Envelope<AccountLogoutResult>>;

        public class CommandHandler : IRequestHandler<Command, Envelope<AccountLogoutResult>>
        {
            private readonly IUserService _userService;
            private readonly IMapper _mapper;
            private readonly ILogger<AccountLoginHandler> _logger;

            public CommandHandler(IUserService userService, IMapper mapper, ILogger<AccountLoginHandler> logger)
            {
                _userService = userService;
                _mapper = mapper;
                _logger = logger;
            }

            public async Task<Envelope<AccountLogoutResult>> Handle(Command command, CancellationToken cancellationToken)
            {
                var logoutResult = await _userService.Logout(command.Token);
                _logger.LogDebug($"Logout succeeded for user '{logoutResult.UserName}'.");

                var result = _mapper.Map<AccountLogoutResult>(logoutResult);
                return Envelope<AccountLogoutResult>.Success(result);
            }
        }
    }
}
