﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using Tuitio.Application.Services;
using Tuitio.PublishedLanguage.Dto;
using MediatR;
using Microsoft.Extensions.Logging;
using System.Threading;
using System.Threading.Tasks;
using Tuitio.PublishedLanguage.Constants;

namespace Tuitio.Application.CommandHandlers
{
    public class AuthorizeTokenHandler
    {
        public record Command(string Token) : IRequest<Envelope<TokenAuthorizationResult>>;

        public class CommandHandler : IRequestHandler<Command, Envelope<TokenAuthorizationResult>>
        {
            private readonly IUserService _userService;
            private readonly IMapper _mapper;
            private readonly ILogger<AuthorizeTokenHandler> _logger;

            public CommandHandler(IUserService userService, IMapper mapper, ILogger<AuthorizeTokenHandler> logger)
            {
                _userService = userService;
                _mapper = mapper;
                _logger = logger;
            }

            public Task<Envelope<TokenAuthorizationResult>> Handle(Command command, CancellationToken cancellationToken)
            {
                var token = _userService.Authorize(command.Token);
                if (token == null)
                {
                    _logger.LogDebug($"Authorization failed for token '{command.Token}'.");
                    var result = Envelope<TokenAuthorizationResult>.Error(EnvelopeStatus.UNAUTHORIZED);
                    return Task.FromResult(result);
                }

                _logger.LogDebug($"Authorization succeeded for token '{command.Token}'.");
                var authorizationResult = new TokenAuthorizationResult(_mapper.Map<TokenCore>(token));
                var envelope = Envelope<TokenAuthorizationResult>.Success(authorizationResult);
                return Task.FromResult(envelope);
            }
        }
    }
}
