﻿// Copyright (c) 2020 Tudor Stanciu

using Tuitio.Domain.Data.DbContexts;
using Tuitio.Domain.Entities;
using Tuitio.Domain.Models;
using Tuitio.Domain.Repositories;
using Microsoft.EntityFrameworkCore;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace Tuitio.Domain.Data.Repositories
{
    class UserRepository : IUserRepository
    {
        private readonly TuitioDbContext _dbContext;

        public UserRepository(TuitioDbContext dbContext)
        {
            _dbContext = dbContext;
        }

        public Task<AppUser> GetUser(string userName, string password)
        {
            return _dbContext.Users
                .Include(z => z.Status)
                .Include(z => z.Claims)
                .FirstOrDefaultAsync(z => z.UserName == userName && z.Password == password);
        }

        public async Task UpdateUserAfterAuthentication(AppUser user, Token token, string tokenRaw)
        {
            var userToken = new UserToken()
            {
                TokenId = token.TokenId,
                UserId = token.UserId,
                Token = tokenRaw,
                ValidFrom = token.CreatedAt,
                ValidUntil = token.CreatedAt.AddMilliseconds(token.ExpiresIn)
            };

            await _dbContext.AddAsync(userToken);

            user.LastLoginDate = DateTime.UtcNow;
            
            //asta nu trebuie
            //_dbContext.Update(user);

            await _dbContext.SaveChangesAsync();
        }

        public Task<UserToken[]> GetActiveTokens()
        {
            var currentDate = DateTime.UtcNow;
            var query = _dbContext.UserTokens
                .Where(z => z.ValidFrom <= currentDate && z.ValidUntil >= currentDate);

            // read all tokens, keep the valid ones and remove the expired ones.
            return query.ToArrayAsync();
        }

        public Task RemoveToken(Guid tokenId)
        {
            var token = new UserToken()
            {
                TokenId = tokenId
            };

            _dbContext.UserTokens.Attach(token);
            _dbContext.UserTokens.Remove(token);

            return _dbContext.SaveChangesAsync();
        }
    }
}
